#!/usr/bin/awk

# This is an AWK script that transforms Synology Diskstation playlists into
# SqeeueBox Server playlists. The input is read from standard in and the
# output is written to standard out. The formats of the playlists are:
#
# Audiostation:
#
##EXTM3U
#...
#../../../../../../volume1/music/Rap/Eminem/The Eminem Show/1-Curtains_up.mp3
#
# SqueezeBox Server
#
##CURTRACK 0
##EXTM3U
##EXTURL:file:///volume1/music/Rap/Eminem/The%20Eminem%20Show/1-Curtains_up.mp3
##EXTINF:<numeric_id>,Curtains up
#/volume1/music/Rap/Eminem/The Eminem Show/1-Curtains_up.mp3
#

# A function that returns the URL-escaped version of the given string
function urlEscape( string )
{
  # This would give us full URL-encoding:
  #cmd="perl -MURI::Escape -e 'print uri_escape($ARGV[0]);' \"" string "\"";
  #result="";
  #cmd | getline result;
  #close(cmd);
  #return result;

  # Apparently, we only need to escape the blank char (see the above example)
  gsub( / /, "%20", string );
  return string;
}

BEGIN{
  # Use "/" as field separator
  FS="/";

  # Write the preamble as indicated above
  print "#CURTRACK 0";
  print "#EXTM3U";
}

/^#/ {
  # Filter out the comment lines
}

/^[^#]/ {
  # Initialization
  absolute_path="";
  trackname="";

  # Process non-comment lines. Those will hold paths to the MP3s
  # Strip away all leading ../ path parts to get the absolute path
  for ( i = 1; i <= NF; i++ ) {
    if ( $i != ".." ) {
      absolute_path=absolute_path "/" $i;
    }
    if ( i == NF ) {
      trackname = $i;
    }
  }

  # Strip away \r characters
  gsub( /\r/, "", absolute_path );                            
  gsub( /\r/, "", trackname ); 

  # Formatted printing
  printf( "#EXTURL:file://%s\n", urlEscape( absolute_path ) );
  printf( "#EXTINF:%d,%s\n", NR, trackname );
  printf( "%s\n", absolute_path );
}
